/*globals require*/
require([
    'jquery',
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/presentation/ui/pricetabledata-test.html',

    'de_epages/presentation/ui/pricetabledata',
    '$ready!'
], function ($, QUnit, fixture) {
    'use strict';

    QUnit.module('de_epages.shop.ui.pricetabledata', {
        teardown: fixture.destroy
    });

    QUnit.fixtureVar = fixture;

    QUnit.asyncTest('Checking pricetabledata\'s methods', 5, function () {
        $.each([
            '_create',
            'updateData',
            'getDataByName',
            'getIndexFilteredDataByName',
            'destroy'
        ], function () {
            QUnit.ok(
                $.isFunction($.ui.presentationUiPriceTableData.prototype[this]),
                this + ' is a method of the widget'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest('Check widget instance', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var pricetable = $('.ep-bulkprices-table_1');

        // instantiate widget
        pricetable.presentationUiPriceTableData();

        window.setTimeout(function() {
            // check existence of widget
            QUnit.ok(
                pricetable.data('uiPresentationUiPriceTableData') instanceof $.Widget,
                'Widget instance exists.'
            );

            // destroy widget
            pricetable.presentationUiPriceTableData('destroy');

            // check that widget was destroyed
            QUnit.ok(
                !(pricetable.data('uiPresentationUiPriceTableData') instanceof $.Widget),
                'Widget instance destroyed.'
            );

            // run test
            QUnit.start();

        }, 300);
    });

    QUnit.asyncTest('Check data functionality of widget', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var pricetable = $('.ep-bulkprices-table_1'),
            initialTestValues = {
                "Quantity": ["5", "4", "6"],
                "Price": ["5", "4", "6"],
                "Discount": ["5", "4", "6"]
            },
            newValue = 4,
            index,
            values,
            testValue,
            input;

        // instantiate widget
        pricetable.presentationUiPriceTableData();

        window.setTimeout(function() {
            // execute *updateData* method to get initial data
            pricetable.presentationUiPriceTableData('updateData');

            // get values directly from the widget
            values = pricetable.data('uiPresentationUiPriceTableData').values;

            // check keys
            $.each(Object.keys(initialTestValues), function (index, key) {
                QUnit.ok(
                    values[key],
                    '"values" object contains property "' + key + '"'
                );
            });

            // check initial values for each key
            $.each(Object.keys(initialTestValues), function (index, key) {
                QUnit.equal(
                    JSON.stringify(values[key]),
                    JSON.stringify(initialTestValues[key]),
                    'Property "' + key + '" contains the correct data: ' + JSON.stringify(initialTestValues[key])
                );
            });


            // test methods *getDataByName* and implicit *updateData*
            // get first value in *Quantity array* from widget formatted with a *#* as first char
            testValue = pricetable.presentationUiPriceTableData('getDataByName', 'Quantity', function(val) {return "#" + val;}).data[0];
            QUnit.equal(
                "#" + initialTestValues['Quantity'][0],
                testValue,
                'First "Quantity" value starts with the character "#" (' + testValue + ').'
            );

            // check first value
            input = pricetable.find('input[name=NewQuantity]:eq(0)');
            // get first value in *Quantity array* from widget
            // executing *getDataByName* method from widget runs *updateData* internaly
            testValue = pricetable.presentationUiPriceTableData('getDataByName', 'Quantity').data[0];
            QUnit.equal(
                input.val(),
                testValue,
                'First "Quantity" input contains the value (' + testValue + ') from the first index of "Quantities" in data array.'
            );


            // change value of input
            input.val(newValue);

            // get first value in *Quantity array* from widget
            // executing *getDataByName* method from widget runs *updateData* internaly
            testValue = pricetable.presentationUiPriceTableData('getDataByName', 'Quantity').data[0];
            // check first value again
            QUnit.equal(
                input.val(),
                testValue,
                'First "Quantity" input contains the value (' + testValue + ') from the first index of "Quantities" in data array.'
            );


            // test method *getIndexFilteredDataByName*
            // filter value at index *0* in *Quantity array* and format *data* array and *value* with *#* as first character
            index = 0;
            testValue = pricetable.presentationUiPriceTableData('getIndexFilteredDataByName', index, 'Quantity', function(val) {return "#" + val;});
            QUnit.equal(
                "#" + initialTestValues['Quantity'][index + 1],
                testValue.data[index],
                'First "Quantity" value was filtered from array. The others starts with the character "#". The first value from the array equals to the second in the test array (' + testValue.data[index] + ').'
            );
            QUnit.equal(
                "#" + newValue,
                testValue.value,
                'Filtered "Quantity" value starts with the character "#" (' + testValue.value + ').'
            );

            // get *Quantity* array without element on index 0
            testValue = pricetable.presentationUiPriceTableData('getIndexFilteredDataByName', 0, 'Quantity');
            QUnit.equal(
                testValue.data.length,
                initialTestValues.Quantity.length - 1,
                'Index filtered "Quantity" array has length ' + (initialTestValues.Quantity.length - 1) + '.'
            );
            // test filtered value
            QUnit.equal(
                testValue.value,
                pricetable.presentationUiPriceTableData('getDataByName', 'Quantity').data[0],
                'Value ' + testValue.value + ' at index 0 from "Quantity" array was filtered.'
            );
            // test filtered array
            QUnit.equal(
                JSON.stringify(testValue.data),
                JSON.stringify(pricetable.presentationUiPriceTableData('getDataByName', 'Quantity').data.slice(1)),
                'Index filtered "Quantity" array is: ' + JSON.stringify(testValue.data) + ' - (filtered from: ' + JSON.stringify(pricetable.presentationUiPriceTableData('getDataByName', 'Quantity').data) + ')'
            );

            // run test
            QUnit.start();

        }, 300);
    });
});